/*
 * File: app/model/Form/Observer.js
 *
 * This file was generated by Sencha Architect
 * http://www.sencha.com/products/architect/
 *
 * This file requires use of the Ext JS 4.2.x library, under independent license.
 * License of Sencha Architect does not include license for Ext JS 4.2.x. For more
 * details see http://www.sencha.com/license or contact license@sencha.com.
 *
 * This file will be auto-generated each and everytime you save your project.
 *
 * Do NOT hand edit this file.
 */

/*
	Encapsulates form states and using Observer Pattern triggers for all subscribers if all form fields are ok or not.
	This is a singleton class, means that it is not instantiable and there is only one instance for use in installwiz.model.Form.Observer
*/
Ext.define('installwiz.model.Form.Observer', {
	extend: 'Ext.data.Model',

	singleton: true,
	requires: [
		'Ext.data.Field'
	],

	fields: [
		{
			defaultValue: {
				email: false,
				password: false,
				repassword: false,
				eula: false
			},
			name: 'states'
		},
		{
			defaultValue: false,
			name: 'lastResult',
			type: 'boolean'
		},
		{
			defaultValue: [
				
			],
			name: 'handlers'
		}
	],

	/*
		@public
		Changes the state of a particular key.
		@param {key} <String> Key representing state
		@param {value} <Bool> | Optional | Key representing state
	*/
	updateState: function(key, value) {
		var me = this;

		installwiz.Maybe.of(me.get("states"))
			.chain(installwiz.Support.prop(key))
			.map(function(flag) {
				var val = installwiz.Maybe.withDefault(!flag, installwiz.Maybe.of(value));
				var fresh = Ext.Object.merge({}, me.get("states"), installwiz.Support.createObjectWith(key, val));
				return (me.set("states", fresh), fresh);
			})
			.map(installwiz.Support.objectToArray)
			.map(installwiz.Support.allReducer)
			.map(me.trigger.bind(me));
	},

	/*
		@public
		Trigger all subscribers if the last result is different.
		@param {stats} <Bool> stats represents a Boolean value of the logical conjunction of all states
	*/
	trigger: function(stats) {
		if (this.get("lastResult") === stats) return undefined;
		this.set("lastResult", stats);
		this.get("handlers").reduce
			( function(acc, f) { return f(stats); }
			, null // null passed, never touched acc variable
			);
	},

	/*
		@public
		Inserts a function in the pipe.
		@param {f} <(Bool -> _)> Only a function that receives a boolean as a parameter, its return is irrelevant
	*/
	subscribe: function(f) {
		if (typeof f !== "function") return console.warn("Not passing a function to subscribe.");
		this.set("handlers", this.get("handlers").concat(f));
	}

});